import asyncio
from semi_ate_testers.master_testers.tester_master_interface import MasterTesterInterface


class DummyMasterParallelTester(MasterTesterInterface):
    def get_sites_count(self):
        # TODO: temporary value for 16
        return 16

    def get_site_states(self) -> list:
        pass

    async def get_site_states(self, timeout: int) -> list:
        await asyncio.sleep(0.0)
        return 16 * [1]

    def release_test_execution(self, sites: list):
        pass

    def get_strategy_type(self):
        return "simple"



from SCT8.tester import Tester as sct8


# TODO: substitute print by logger, see issue #161


class MiniSCT(sct8):

    def get_sites_count(self):
        return 1

    def do_request(self, site_id: int, timeout: int) -> bool:
        return True

    def test_in_progress(self, site_id: int):
        print(f'Tester.test_in_progress({site_id})')

    def test_done(self, site_id: int, timeout: int):
        print(f'Tester.test_done({site_id})')

    def do_init_state(self, site_id: int):
        print(f'Tester.do_init_state({site_id})')




from abc import ABC, abstractmethod


class MasterTesterInterface(ABC):
    @abstractmethod
    def get_sites_count(self):
        pass

    @abstractmethod
    def get_site_states(self) -> list:
        pass

    @abstractmethod
    def release_test_execution(self, sites: list):
        pass

    @abstractmethod
    def get_strategy_type(self):
        return "default"



import requests
import json


class Flatcache:
    def __init__(self):
        self.target_ip = ""
        self.target_port = ""
        self.last_part_id = ""
        self.fetched_data = {}
        self.subdoc_cache = {}

    def apply_configuration(self, data: dict):
        self.target_ip = data["ip"]
        self.target_port = data["port"]

        # Canary request -> check if service is available (i.e. requests succeeds)
        # and if it reports the correct API version.
        url = f"http://{self.target_ip}:{self.target_port}/api"
        r = requests.get(url)
        api_version = json.loads(r.content)
        version = api_version["version"]
        if version != 1:
            raise ValueError(f"Expected flatcache api version to be 1, but is {version}")

    def get_value(self, part_id: str, value_name: str) -> dict:
        if self.last_part_id != part_id:
            self.do_fetch(part_id)

        # The valuename is composed of <programname>.<testinstancename>.<paramname>
        # the programname is used as catflache subdoc id, so we split that off to
        # get the right subdoc:
        value_parts = value_name.split(".", 1)
        subdocid = value_parts[0]
        normalized_value_name = value_parts[1]

        found_subdoc = self.fetched_data.get(subdocid)

        if found_subdoc is None:
            return None

        for record in found_subdoc:
            record_type = record["type"]
            if record_type == "PTR":
                if record["TEST_TXT"] == normalized_value_name:
                    return record

            if record_type == "PIR" and value_name == "PIR":
                return record

            if record_type == "PRR" and value_name == "PRR":
                return record

    def get_cached_value(self, value_name: str) -> dict:
        return self.get_value(self.last_part_id, value_name)

    def do_fetch(self, part_id):
        url = f"http://{self.target_ip}:{self.target_port}/{part_id}"
        r = requests.get(url)
        try:
            values = json.loads(r.content)
            self.last_part_id = part_id
            self.fetched_data = {}
            for entry in values["contents"]:
                fetched_data = json.loads(entry["contents"])
                subdoc_id = entry["subdocid"]
                self.fetched_data[subdoc_id] = fetched_data

        except Exception as e:
            print(e)
            print(r.content)

    def publish(self, part_id: str, program_name: str, data):
        # Trouble: This will completely replace the date stored in the cache
        #  with "data" -> we don't want that.
        url = f"http://{self.target_ip}:{self.target_port}/{part_id}"
        requests.put(url, json={"subdocid": str(program_name), "contents": json.dumps(data)})

    def drop_part(self, part_id: str):
        url = f"http://{self.target_ip}:{self.target_port}/{part_id}"
        requests.delete(url)



from semi_ate_testers.testers.tester_interface import TesterInterface

class DummyParallelTester(TesterInterface):
    def get_sites_count(self):
        # TODO: temporary value for 16
        return 16

    def do_request(self, site_id: int, timeout: int) -> bool:
        return True

    def test_in_progress(self, site_id: int):
        pass

    def test_done(self, site_id: int, timeout: int):
        pass

    def do_init_state(self, site_id: int):
        pass



from semi_ate_testers.testers.tester_interface import TesterInterface

class DummySingleTester(TesterInterface):
    def pulse_trigger_out(self, pulse_width_ms):
        # ToDo: Implement with actual hardware.
        print(f"Single Tester: Pulse Trigger Out")

    def get_sites_count(self):
        return 1

    def do_request(self, site_id: int, timeout: int) -> bool:
        return True

    def test_in_progress(self, site_id: int):
        pass

    def test_done(self, site_id: int, timeout: int):
        pass

    def do_init_state(self, site_id: int):
        pass



import asyncio
from semi_ate_testers.master_testers.tester_master_interface import MasterTesterInterface


class DummyMasterSingleTester(MasterTesterInterface):
    def pulse_trigger_out(self, pulse_width_ms):
        # ToDo: Implement with actual hardware.
        print(f"Single Tester: Pulse Trigger Out")

    def get_sites_count(self):
        return 1

    async def get_site_states(self, timeout: int) -> list:
        await asyncio.sleep(0.0)
        return 16 * [1]

    def release_test_execution(self, sites: list):
        pass

    def get_strategy_type(self):
        return "simple"



from setuptools import find_packages, setup
from pathlib import Path
from semi_ate_testers import __version__

version = __version__
requirements_path = Path(Path(__file__).parents[0], 'requirements/run.txt')
       
with requirements_path.open('r') as f:
    install_requires = list(f)

readme_path = Path(Path(__file__).parent, './semi_ate_testers/README.md')
with readme_path.open('r') as f:
    long_description = f.read()

setup( 
    name="semi-ate-testers",
    version=version,
    description='semi-ate-testers is an example implementation of a plugin that can be consumed by the semi-ate-spyder plugin and the master application.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    author="The Semi-ATE Project Contributors",
    author_email="ate.organization@gmail.com",
    packages=find_packages(),
    install_requires=install_requires,
    include_package_data=True,
    entry_points={"ate.org": ["plug = semi_ate_testers:Plugin"]},
    py_modules=["semi_ate_testers"],
)
